import { Db, MongoClient, ObjectId } from 'mongodb';

const fs = require('fs');

module.exports = (on: any, config: any) => {
  on('after:spec', (spec: Cypress.Spec, results: CypressCommandLine.RunResult) => {
    if (results && results.video && results.stats.failures === 0) {
      console.log(`Delete recorded video because spec passed: ${results.video}`);
      fs.unlinkSync(results.video);
    }
  });

  // Workaround für Angular 13 und Cypress mit Webpack 4,
  // Siehe https://github.com/cypress-io/cypress/issues/19066#issuecomment-1012055705
  // Entfernen, sobald Cypress Webpack 5 nutzt - https://github.com/cypress-io/cypress/issues/19555
  // Ursache: Angular linker needed to link partial-ivy code,
  //   see https://angular.io/guide/creating-libraries#consuming-partial-ivy-code-outside-the-angular-cli
  // Fehlerbild:
  //   - Anwendung läuft im Browser, aber nicht in Cypress.
  //   - Fehlermeldung in Cypress: The injectable 'SystemDateTimeProvider' needs to be compiled using the JIT compiler, but '@angular/compiler' is not available.
  // Lösung:
  //   - NPM-Paket identifizieren, dass "SystemDateTimeProvider" enthält.
  //   - NPM-Paket im "test" Attribut unten hinzufügen.
  const webpackPreprocessor = require('@cypress/webpack-batteries-included-preprocessor');
  const webpackOptions = webpackPreprocessor.defaultOptions.webpackOptions;

  webpackOptions.module.rules.unshift({
    test: /[/\\](@angular|@ngxp|angular-oauth2-oidc)[/\\].+\.m?js$/,
    resolve: {
      fullySpecified: false,
    },
    use: {
      loader: 'babel-loader',
      options: {
        plugins: ['@angular/compiler-cli/linker/babel'],
        compact: false,
        cacheDirectory: true,
      },
    },
  });

  on(
    'file:preprocessor',
    webpackPreprocessor({
      webpackOptions: webpackOptions,
      typescript: require.resolve('typescript'),
    }),
  );

  on('task', {
    dropCollections(collections) {
      console.log('dropCollections: ', collections);
      dropCollectionsFromDatabase(config, collections);
      return 0;
    },
    initOrganisationsEinheitData({ collection, data }) {
      console.log('initOrganisationsEinheitData');
      insertIntoDatabase(config, collection, parseOrganisationsEinheitData(data));
      return 0;
    },
  });
  return config;
  // Ende - Workaround für Angular 13 und Cypress mit Webpack 4
};

function dropCollectionsFromDatabase(config, collections) {
  dropCollections(getDatabaseUrl(config), getDatabase(config), collections);
}

function getDatabaseUrl(config) {
  return config.env.dbUrl;
}

function getDatabase(config) {
  return config.env.database;
}

function dropCollections(databaseUrl, databaseName, collections) {
  MongoClient.connect(databaseUrl, (error, connection) => {
    if (!error) {
      const db = connection.db(databaseName);
      collections.forEach((oneCollection, index) => {
        console.log('drop collection', oneCollection);
        db.collection(oneCollection).drop(() => {
          //CHECKME Ist die Abfrage notwendig?
          if (index == collections.length) {
            console.log('close connection');
            connection.close();
          }
        });
      });
    } else {
      console.error('Error: ', error);
    }
  });
}

function insertIntoDatabase(config, collection, data) {
  insert(getDatabaseUrl(config), getDatabase(config), collection, data);
}

function insert(databaseUrl, databaseName, collection, data) {
  MongoClient.connect(databaseUrl, (error, connection) => {
    console.log(`connect to ${databaseName} database with ${databaseUrl}`);
    if (!error) {
      console.log('success');
      const db: Db = connection.db(databaseName);

      db.collection(collection).drop(() => {
        db.createCollection(collection, (error) => handleCreateCollection(db, connection, collection, data, error));
      });
    } else {
      console.error('Error: ', error);
    }
  });
}

function handleCreateCollection(db, connection, collection, data, error) {
  if (error) {
    console.error(`Fehler beim Erstellen der Collection "${collection}": `, error);
  } else {
    console.log(`Collection ${collection} erfolgreich erstellt`);
    insertManyToDatabase(db, connection, collection, data);
  }
}

function insertManyToDatabase(db, connection, collection, data) {
  db.collection(collection).insertMany(data, (error) => handleInsertMany(connection, error));
}

function handleInsertMany(connection, error) {
  if (error) {
    console.error('Fehler beim Einlesen der Daten: ', error);
  } else {
    console.log('Die Daten wurden erfolgreich eingelesen.');
  }
  connection.close();
}

function parseOrganisationsEinheit(organisationsEinheit) {
  organisationsEinheit._id = createObjectId(organisationsEinheit);
}

function createObjectId(field) {
  return new ObjectId(field._id.$oid);
}

function parseOrganisationsEinheitData(data) {
  data.forEach((organisationsEinheit) => parseOrganisationsEinheit(organisationsEinheit));
  return data;
}
