import { createEmptyStateResource, createStateResource, doIfLoadingRequired, StateResource } from '@alfa-client/tech-shared';
import { BehaviorSubject, first, map, Observable, startWith, tap } from 'rxjs';

export abstract class KeycloakResourceService<T> {
  readonly stateResource: BehaviorSubject<StateResource<T[]>> = new BehaviorSubject(createEmptyStateResource());

  public get(): Observable<StateResource<T[]>> {
    return this.stateResource.asObservable().pipe(tap((stateResource) => this.handleChanges(stateResource)));
  }

  handleChanges(stateResource: StateResource<T[]>) {
    doIfLoadingRequired(stateResource, () => this.loadResource());
  }

  loadResource(): void {
    this.setLoading();
    this.getItemsFromKeycloak()
      .pipe(first())
      .subscribe((items) => this.updateResource(items));
  }

  abstract getItemsFromKeycloak(): Observable<T[]>;

  private updateResource(items: T[]): void {
    this.stateResource.next(createStateResource(items));
  }

  public create(item: Partial<T>): Observable<boolean> {
    return this.handleLoading(this.createInKeycloak(item));
  }

  abstract createInKeycloak(item: Partial<T>): Observable<T>;

  public save(item: T): Observable<boolean> {
    return this.handleLoading(this.saveInKeycloak(item));
  }

  abstract saveInKeycloak(item: T): Observable<void>;

  public delete(id: string): Observable<boolean> {
    return this.handleLoading(this.deleteInKeycloak(id));
  }

  abstract deleteInKeycloak(id: string): Observable<void>;

  handleLoading(action: Observable<unknown>): Observable<boolean> {
    this.setLoading();
    return this.progress(this.refreshAfterFirstEmit(action));
  }

  refreshAfterFirstEmit(action: Observable<unknown>): Observable<unknown> {
    return action.pipe(
      first(),
      tap(() => this.refresh()),
    );
  }

  progress(action: Observable<unknown>): Observable<boolean> {
    return action.pipe(
      map(() => false),
      startWith(true),
    );
  }

  setLoading(loading: boolean = true): void {
    this.stateResource.next({
      ...this.stateResource.value,
      loading,
    });
  }

  refresh(): void {
    this.stateResource.next({
      ...createEmptyStateResource(),
      reload: true,
    });
  }

  public selectResource(): Observable<StateResource<T[]>> {
    return this.stateResource.asObservable();
  }
}
