import { BescheidResource, BescheidService, BescheidWizardDialogResult, BescheidWizardStep } from '@alfa-client/bescheid-shared';
import { CommandOrder, CommandResource, isSuccessfulDone } from '@alfa-client/command-shared';
import {
  StateResource,
  createEmptyStateResource,
  hasStateResourceError,
  isEscapeKey,
  isLoaded,
  isNotLoading,
  isNotNil,
} from '@alfa-client/tech-shared';
import { OzgcloudDialogService } from '@alfa-client/ui';
import { VorgangWithEingangLinkRel, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { DIALOG_DATA, DialogRef } from '@angular/cdk/dialog';
import { Component, Inject, OnDestroy, OnInit, ViewContainerRef } from '@angular/core';
import { Resource, hasLink } from '@ngxp/rest';
import { Observable, Subscription, filter, first, of, switchMap, tap } from 'rxjs';
import {
  BescheidWizardCancelDialogContainerComponent,
  CancelWizardDialogData,
  CancelWizardDialogResult,
} from './bescheid-wizard/cancel-dialog-container/bescheid-wizard-cancel-dialog-container.component';
import { BescheidFormService } from './bescheid.formservice';

export interface BescheidWizardDialogData {
  vorgangWithEingangResource: VorgangWithEingangResource;
}

@Component({
  selector: 'alfa-bescheid-wizard-container',
  templateUrl: './bescheid-wizard-container.component.html',
  providers: [BescheidFormService],
})
export class BescheidWizardContainerComponent implements OnInit, OnDestroy {
  public bescheidDraftStateResource$: Observable<StateResource<BescheidResource>> =
    of(createEmptyStateResource<BescheidResource>());
  public submitStateResource$: Observable<StateResource<Resource>> = of(createEmptyStateResource<Resource>());

  cancelDialogRef: DialogRef<CancelWizardDialogResult>;
  vorgangWithEingangResource: VorgangWithEingangResource;
  keydownEventsSubscription: Subscription;

  constructor(
    @Inject(DIALOG_DATA) private readonly dialogData: BescheidWizardDialogData,
    private readonly dialogRef: DialogRef<BescheidWizardDialogResult>,
    readonly viewContainerRef: ViewContainerRef,
    public readonly formService: BescheidFormService,
    readonly bescheidService: BescheidService,
    private readonly ozgcloudDialogService: OzgcloudDialogService,
  ) {
    this.vorgangWithEingangResource = dialogData.vorgangWithEingangResource;
  }

  ngOnInit(): void {
    this.bescheidService.init();
    this.formService.setVorgangWithEingangResource(this.vorgangWithEingangResource);
    this.subscribeToBescheidResourceIfExists();
    this.handleEscapeKey();
  }

  ngOnDestroy(): void {
    this.keydownEventsSubscription.unsubscribe();
  }

  subscribeToBescheidResourceIfExists(): void {
    if (hasLink(this.vorgangWithEingangResource, VorgangWithEingangLinkRel.BESCHEID_DRAFT)) {
      this.bescheidDraftStateResource$ = this.bescheidService.getBescheidDraft();
    }
  }

  handleEscapeKey(): void {
    this.keydownEventsSubscription = this.dialogRef.keydownEvents
      .pipe(
        filter(isEscapeKey),
        switchMap(() => this.bescheidDraftStateResource$),
        filter(isNotLoading),
      )
      .subscribe((bescheidStateResource: StateResource<BescheidResource>) => {
        this.cancelWizard(bescheidStateResource.resource);
      });
  }

  public onWeiter(nextStep: BescheidWizardStep): void {
    this.submitStateResource$ = this.formService.submit().pipe(
      switchMap((commandStateResource: StateResource<CommandResource>) => {
        if (this.isBescheidSuccessfullyCreated(commandStateResource.resource)) {
          this.bescheidDraftStateResource$ = this.bescheidService.getBescheidDraft();
          return this.bescheidDraftStateResource$;
        }
        return of(commandStateResource);
      }),
      tap((stateResource: StateResource<Resource>) => {
        if (isLoaded(stateResource) && !hasStateResourceError(stateResource)) {
          this.bescheidService.setActiveStep(nextStep);
        }
      }),
    );
  }

  isBescheidSuccessfullyCreated(commandResource: CommandResource): boolean {
    return isSuccessfulDone(commandResource) && commandResource.order === CommandOrder.CREATE_BESCHEID;
  }

  public cancelWizard(bescheidResource: BescheidResource): void {
    if (isNotNil(this.cancelDialogRef)) return;
    this.openCancelDialog(bescheidResource);
    this.subscribeToCancelDialogClosed();
  }

  openCancelDialog(bescheidResource: BescheidResource) {
    const dialogData: CancelWizardDialogData = {
      bescheidResource,
    };

    this.cancelDialogRef = this.ozgcloudDialogService.openInCallingComponentContext(
      BescheidWizardCancelDialogContainerComponent,
      this.viewContainerRef,
      dialogData,
    );
  }

  subscribeToCancelDialogClosed() {
    this.cancelDialogRef.closed
      .pipe(first())
      .subscribe((result: CancelWizardDialogResult) => this.handleCancelDialogClosed(result));
  }

  handleCancelDialogClosed(closeResult: CancelWizardDialogResult): void {
    this.cancelDialogRef = null;
    if (isNotNil(closeResult) && closeResult.closeWizard) {
      this.dialogRef.close({ reloadVorgang: true });
    }
  }

  public onVorgangAbgeschlossen(): void {
    this.dialogRef.close();
  }
}
