import { Bescheid, BescheidLinkRel, BescheidResource, BescheidSendBy, BescheidService } from '@alfa-client/bescheid-shared';
import { formatForDatabase } from '@alfa-client/tech-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import { UntypedFormBuilder } from '@angular/forms';
import { faker } from '@faker-js/faker';
import { ResourceUri } from '@ngxp/rest';
import { EMPTY, Observable, of } from 'rxjs';
import { createBescheid, createBescheidResource } from '../../../../bescheid-shared/src/test/bescheid';
import { singleCold } from '../../../../tech-shared/test/marbles';
import { createVorgangWithEingangResource } from '../../../../vorgang-shared/test/vorgang';
import { BescheidFormService } from './bescheid.formservice';

registerLocaleData(localeDe);

describe('BescheidFormService', () => {
  let service: BescheidFormService;
  let bescheidService: Mock<BescheidService>;
  const now: Date = new Date();
  Date.now = jest.fn().mockReturnValue(now);
  const vorgangWithEingangResource: VorgangWithEingangResource = createVorgangWithEingangResource();

  beforeEach(() => {
    bescheidService = mock(BescheidService);
    bescheidService.createBescheid.mockReturnValue(of(EMPTY));
    service = new BescheidFormService(new UntypedFormBuilder(), useFromMock(bescheidService));
    service.setVorgangWithEingangResource(vorgangWithEingangResource);
  });

  describe('doSubmit', () => {
    it('should create bescheid', () => {
      const formValue: Bescheid = createBescheid();
      service.getBescheidFormValue = jest.fn().mockReturnValue(formValue);

      service.submit();

      expect(bescheidService.createBescheid).toHaveBeenCalledWith(vorgangWithEingangResource, formValue);
    });
  });

  describe('patchValues', () => {
    let bescheidResource: BescheidResource;
    let patch: jest.Mock;

    beforeEach(() => {
      bescheidResource = createBescheidResource();
      patch = service.patch = jest.fn();
    });

    it('should patch', () => {
      service.patchValues(bescheidResource);

      expect(patch).toHaveBeenCalledWith({
        [BescheidFormService.FIELD_BESCHIEDEN_AM]: bescheidResource.beschiedenAm,
        [BescheidFormService.FIELD_BEWILLIGT]: String(bescheidResource.bewilligt),
        [BescheidFormService.FIELD_BESCHEID_DOCUMENT]: null,
        [BescheidFormService.FIELD_SEND_BY]: String(bescheidResource.sendBy),
        [BescheidFormService.FIELD_NACHRICHT_SUBJECT]: bescheidResource.nachrichtSubject,
        [BescheidFormService.FIELD_NACHRICHT_TEXT]: bescheidResource.nachrichtText,
      });
    });

    it('should patch sendBy to default', () => {
      service.patchValues({
        ...bescheidResource,
        sendBy: undefined,
      });

      expect(patch).toHaveBeenCalledWith({
        [BescheidFormService.FIELD_BESCHIEDEN_AM]: bescheidResource.beschiedenAm,
        [BescheidFormService.FIELD_BEWILLIGT]: String(bescheidResource.bewilligt),
        [BescheidFormService.FIELD_SEND_BY]: BescheidSendBy.NACHRICHT,
        [BescheidFormService.FIELD_BESCHEID_DOCUMENT]: null,
        [BescheidFormService.FIELD_NACHRICHT_SUBJECT]: bescheidResource.nachrichtSubject,
        [BescheidFormService.FIELD_NACHRICHT_TEXT]: bescheidResource.nachrichtText,
      });
    });

    it('should patch bescheid document uri', () => {
      const bescheidDocumentUri: ResourceUri = faker.internet.url();
      service.patchValues({
        ...bescheidResource,
        _links: {
          ...bescheidResource._links,
          [BescheidLinkRel.BESCHEID_DOCUMENT]: { href: bescheidDocumentUri },
        },
      });

      expect(patch).toHaveBeenCalledWith({
        [BescheidFormService.FIELD_BESCHIEDEN_AM]: bescheidResource.beschiedenAm,
        [BescheidFormService.FIELD_BEWILLIGT]: String(bescheidResource.bewilligt),
        [BescheidFormService.FIELD_SEND_BY]: BescheidSendBy.NACHRICHT,
        [BescheidFormService.FIELD_BESCHEID_DOCUMENT]: bescheidDocumentUri,
        [BescheidFormService.FIELD_NACHRICHT_SUBJECT]: bescheidResource.nachrichtSubject,
        [BescheidFormService.FIELD_NACHRICHT_TEXT]: bescheidResource.nachrichtText,
      });
    });

    it('should patch attachments', () => {
      service.patchValues(bescheidResource);

      expect(service.getFormValue().attachments).toEqual(bescheidResource.attachments);
    });
  });

  describe('getBescheidFormValueChanges', () => {
    const bescheid: Bescheid = createBescheid();

    beforeEach(() => {
      service.getBescheidFormValue = jest.fn().mockReturnValue(bescheid);
    });

    it('should emit bescheid', () => {
      const bescheidFormValueChanges$: Observable<Bescheid> = service.getBescheidFormValueChanges();

      expect(bescheidFormValueChanges$).toBeObservable(singleCold(bescheid));
    });
  });

  describe('getBescheidFormValue', () => {
    beforeEach(() => {
      service.getFormValue = jest.fn().mockReturnValue({ beschiedenAm: now, bewilligt: 'true' });
    });

    it('should getFormValue', () => {
      service.getBescheidFormValue();

      expect(service.getFormValue).toHaveBeenCalled();
    });

    it('should return bescheid', () => {
      const bescheid: Bescheid = service.getBescheidFormValue();

      expect(bescheid).toEqual({
        bewilligt: true,
        beschiedenAm: formatForDatabase(now),
      } as Bescheid);
    });
  });
});
