import { Bescheid, BescheidLinkRel, BescheidResource, BescheidSendBy, BescheidService } from '@alfa-client/bescheid-shared';
import { AbstractFormService, StateResource, convertToBoolean, formatForDatabase } from '@alfa-client/tech-shared';
import { VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { Injectable } from '@angular/core';
import { UntypedFormArray, UntypedFormBuilder, UntypedFormControl, UntypedFormGroup } from '@angular/forms';
import { Resource, ResourceUri, getUrl, hasLink } from '@ngxp/rest';
import { isUndefined } from 'lodash-es';
import { Observable, startWith } from 'rxjs';

@Injectable()
export class BescheidFormService extends AbstractFormService {
  static readonly FIELD_BESCHIEDEN_AM = 'beschiedenAm';
  static readonly FIELD_BEWILLIGT = 'bewilligt';
  static readonly FIELD_BESCHEID_DOCUMENT = 'bescheidDocument';
  static readonly FIELD_ATTACHMENTS = 'attachments';
  public static readonly FIELD_SEND_BY = 'sendBy';
  static readonly FIELD_NACHRICHT_SUBJECT = 'nachrichtSubject';
  static readonly FIELD_NACHRICHT_TEXT = 'nachrichtText';

  static readonly FIELD_PATH_PREFIX = 'command.body';

  private vorgangWithEingangResource: VorgangWithEingangResource;

  constructor(
    formBuilder: UntypedFormBuilder,
    private readonly bescheidService: BescheidService,
  ) {
    super(formBuilder);
  }

  protected initForm(): UntypedFormGroup {
    return this.formBuilder.group({
      [BescheidFormService.FIELD_BESCHIEDEN_AM]: new UntypedFormControl(new Date(Date.now())),
      [BescheidFormService.FIELD_BEWILLIGT]: new UntypedFormControl('true'),
      [BescheidFormService.FIELD_SEND_BY]: new UntypedFormControl(BescheidSendBy.NACHRICHT),
      [BescheidFormService.FIELD_BESCHEID_DOCUMENT]: new UntypedFormControl(null),
      [BescheidFormService.FIELD_ATTACHMENTS]: new UntypedFormArray([]),
      [BescheidFormService.FIELD_NACHRICHT_SUBJECT]: new UntypedFormControl(''),
      [BescheidFormService.FIELD_NACHRICHT_TEXT]: new UntypedFormControl(''),
    });
  }

  protected doSubmit(): Observable<StateResource<Resource>> {
    return this.bescheidService.createBescheid(this.vorgangWithEingangResource, this.getBescheidFormValue());
  }

  protected getPathPrefix(): string {
    return BescheidFormService.FIELD_PATH_PREFIX;
  }

  public patchValues(bescheid: BescheidResource): void {
    const bescheidDocumentUri: ResourceUri = this.getBescheidDocumentUri(bescheid);
    this.bescheidService.setDocumentUri(bescheidDocumentUri);
    this.patch({
      [BescheidFormService.FIELD_BESCHIEDEN_AM]: bescheid.beschiedenAm,
      [BescheidFormService.FIELD_BEWILLIGT]: String(bescheid.bewilligt),
      [BescheidFormService.FIELD_BESCHEID_DOCUMENT]: bescheidDocumentUri,
      [BescheidFormService.FIELD_SEND_BY]: isUndefined(bescheid.sendBy) ? BescheidSendBy.NACHRICHT : bescheid.sendBy,
      [BescheidFormService.FIELD_NACHRICHT_SUBJECT]: bescheid.nachrichtSubject,
      [BescheidFormService.FIELD_NACHRICHT_TEXT]: bescheid.nachrichtText,
    });
    bescheid.attachments.forEach((attachmentLink) =>
      (this.form.controls[BescheidFormService.FIELD_ATTACHMENTS] as UntypedFormArray).push(
        new UntypedFormControl(attachmentLink),
      ),
    );
  }

  private getBescheidDocumentUri(bescheid: BescheidResource): ResourceUri | null {
    if (hasLink(bescheid, BescheidLinkRel.BESCHEID_DOCUMENT)) {
      return getUrl(bescheid, BescheidLinkRel.BESCHEID_DOCUMENT);
    }
    return null;
  }

  public getBescheidFormValueChanges(): Observable<Bescheid> {
    return this.form.valueChanges.pipe(startWith(this.getBescheidFormValue()));
  }

  public getBescheidFormValue(): Bescheid {
    const value: any = this.getFormValue();
    return {
      ...value,
      beschiedenAm: formatForDatabase(value.beschiedenAm),
      bewilligt: convertToBoolean(value.bewilligt),
      attachments: value.attachments,
    };
  }

  public setVorgangWithEingangResource(vorgangWithEingangResource: VorgangWithEingangResource): void {
    this.vorgangWithEingangResource = vorgangWithEingangResource;
  }
}
