/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { TestBed } from '@angular/core/testing';
import { faker } from '@faker-js/faker';
import {
  ApiError,
  ApiErrorAction,
  TypedActionCreator,
  TypedActionCreatorWithProps,
} from '@alfa-client/tech-shared';
import { Mock, mock } from '@alfa-client/test-utils';
import { provideMockActions } from '@ngrx/effects/testing';
import { Action, createAction, props } from '@ngrx/store';
import { provideMockStore } from '@ngrx/store/testing';
import { Resource, ResourceUri } from '@ngxp/rest';
import { hot } from 'jasmine-marbles';
import { cold } from 'jest-marbles';
import { ColdObservable } from 'jest-marbles/typings/src/rxjs/cold-observable';
import {
  createBinaryFileListResource,
  createBinaryFileResource,
} from 'libs/binary-file-shared/test/binary-file';
import { createApiError } from 'libs/tech-shared/test/error';
import { Observable, of } from 'rxjs';
import { BinaryFileListResource } from '../binary-file.model';
import { BinaryFileRepository } from '../binary-file.repository';
import {
  DownloadBinaryFileAsPdfAction,
  LoadBinaryFileListProps,
  LoadBinaryFileListSuccessProps,
} from './binary-file.actions';
import { BinaryFileEffects } from './binary-file.effects';

import * as BinaryFileActions from './binary-file.actions';

describe('BinaryFileEffects', () => {
  let actions: Observable<Action>;
  let effects: BinaryFileEffects;

  const repository: Mock<BinaryFileRepository> = mock(BinaryFileRepository);

  beforeEach(() => {
    TestBed.configureTestingModule({
      providers: [
        BinaryFileEffects,
        provideMockActions(() => actions),
        provideMockStore(),
        {
          provide: BinaryFileRepository,
          useValue: repository,
        },
      ],
    });

    effects = TestBed.inject(BinaryFileEffects);
  });

  describe('downloadPdf', () => {
    const uri: ResourceUri = faker.internet.url();
    const fileName: string = faker.string.alphanumeric();
    const fileData: Blob = new Blob();

    const downloadAsPdfSuccess: TypedActionCreator = createAction('[Test Action] Download Success');
    const downloadAsPdfFailure: TypedActionCreatorWithProps<ApiErrorAction> = createAction(
      '[Test Action] Download Failure',
      props<ApiErrorAction>(),
    );

    const downloadPdfAction: DownloadBinaryFileAsPdfAction & Action<string> =
      BinaryFileActions.downloadPdf({
        successAction: downloadAsPdfSuccess,
        failureAction: (apiError) => downloadAsPdfFailure({ apiError }),
        fileName,
        uri,
      });

    it('should call repository', () => {
      actions = hot('-a', { a: downloadPdfAction });

      effects.downloadPdf$.subscribe(() => {
        expect(repository.downloadPdf).toHaveBeenCalledWith(uri);
      });
    });

    it('should dispatch success action on no error', () => {
      repository.downloadPdf.mockReturnValue(of(fileData));

      actions = hot('-a', { a: downloadPdfAction });

      const expected: ColdObservable = cold('-(bc)', {
        b: BinaryFileActions.saveFile({ fileName, fileData }),
        c: downloadAsPdfSuccess(),
      });
      expect(effects.downloadPdf$).toBeObservable(expected);
    });

    it.skip('should dispatch failure actions on error', () => {
      const apiError: ApiError = createApiError();
      const error = { error: { error: apiError } };
      const errorResponse = cold('-#', {}, error);
      repository.downloadPdf = jest.fn(() => errorResponse);

      actions = hot('-a', { a: downloadPdfAction });

      const expected: ColdObservable = cold('--b', { b: downloadAsPdfFailure({ apiError }) });
      expect(effects.downloadPdf$).toBeObservable(expected);
    });
  });

  describe('saveAsPdf', () => {
    const fileName: string = faker.string.alphanumeric();
    const fileData: Blob = new Blob();

    it('should save file', () => {
      effects.save = jest.fn();

      actions = hot('a', { a: BinaryFileActions.saveFile({ fileData, fileName }) });

      effects.saveFile$.subscribe(() => {
        expect(effects.save).toHaveBeenCalledWith(fileData, fileName);
      });
    });
  });

  describe('loadBinaryFileList', () => {
    const resource: Resource = createBinaryFileResource();
    const linkRel: string = faker.internet.url();

    const loadSuccess: TypedActionCreatorWithProps<LoadBinaryFileListSuccessProps> = createAction(
      '[Test Action] Download Success',
      props<LoadBinaryFileListSuccessProps>(),
    );
    const loadFailure: TypedActionCreatorWithProps<ApiErrorAction> = createAction(
      '[Test Action] Download Failure',
      props<ApiErrorAction>(),
    );

    const loadBinaryFileListAction: LoadBinaryFileListProps & Action<string> =
      BinaryFileActions.loadBinaryFileList({
        resource,
        linkRel,
        successAction: (binaryFileList: BinaryFileListResource) => loadSuccess({ binaryFileList }),
        failureAction: (apiError: ApiError) => loadFailure({ apiError }),
      });

    const binaryFileList: BinaryFileListResource = createBinaryFileListResource();

    beforeEach(() => {
      repository.getFiles.mockReturnValue(of(binaryFileList));
    });

    it('should call repository', () => {
      actions = of(loadBinaryFileListAction);

      effects.loadBinaryFileList$.subscribe();

      expect(repository.getFiles).toHaveBeenCalledWith(resource, linkRel);
    });

    it('should dispatch success action on no error', () => {
      actions = hot('-a', { a: loadBinaryFileListAction });

      const expected: ColdObservable = cold('-b', { b: loadSuccess({ binaryFileList }) });
      expect(effects.loadBinaryFileList$).toBeObservable(expected);
    });

    it.skip('should dispatch failure actions on error', () => {
      const apiError: ApiError = createApiError();
      const error = { error: { error: apiError } };
      const errorResponse = cold('-#', {}, error);
      repository.getFiles = jest.fn(() => errorResponse);

      const expected: ColdObservable = cold('--b', { b: loadFailure({ apiError }) });
      actions = hot('-a', { a: loadBinaryFileListAction });

      expect(effects.loadBinaryFileList$).toBeObservable(expected);
    });
  });
});
