import { CommandOrder, CommandResource } from '@alfa-client/command-shared';
import {
  existsAsHtmlElement,
  getMockComponent,
  notExistsAsHtmlElement,
} from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { faker } from '@faker-js/faker';
import { createBescheid } from 'libs/bescheid-shared/src/test/bescheid';
import { createCommandResource } from 'libs/command-shared/test/command';
import { getDataTestClassOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { ExpansionPanelWithUserComponent } from '../expansion-panel-with-user/expansion-panel-with-user.component';
import { HistorieItemAttachmentComponent } from '../historie-item-attachment/historie-item-attachment.component';
import { HistorieItemHeaderComponent } from '../historie-item-header/historie-item-header.component';
import { HistorieItemBescheidStatusComponent } from './historie-item-bescheid-status/historie-item-bescheid-status.component';
import { HistorieItemBescheidComponent } from './historie-item-bescheid.component';

describe('HistorieItemBescheidComponent', () => {
  let component: HistorieItemBescheidComponent;
  let fixture: ComponentFixture<HistorieItemBescheidComponent>;

  const expansionPanel: string = getDataTestClassOf('historie-item-bescheid-user-expansion-panel');
  const status: string = getDataTestClassOf('historie-item-bescheid-status');
  const document: string = getDataTestClassOf('historie-item-bescheid-document');
  const attachments: string = getDataTestClassOf('historie-item-bescheid-attachments');
  const header: string = getDataTestClassOf('historie-item-bescheid-header');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        HistorieItemBescheidComponent,
        MockComponent(HistorieItemHeaderComponent),
        MockComponent(HistorieItemBescheidStatusComponent),
        MockComponent(HistorieItemAttachmentComponent),
        MockComponent(ExpansionPanelWithUserComponent),
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(HistorieItemBescheidComponent);
    component = fixture.componentInstance;
    component.command = createCommandResource();
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    describe('headline', () => {
      it('should set for SEND_BESCHEID', () => {
        component.command = { ...createCommandResource(), order: CommandOrder.SEND_BESCHEID };

        component.ngOnInit();

        expect(component.headline).toBe('einen Bescheid per Nachricht verschickt.');
      });

      it('should set for UPDATE_BESCHEID', () => {
        component.command = { ...createCommandResource(), order: CommandOrder.UPDATE_BESCHEID };

        component.ngOnInit();

        expect(component.headline).toBe('einen Bescheid erstellt.');
      });
    });

    describe('has body', () => {
      it('should set true on UPDATE_BESCHEID', () => {
        component.command = { ...createCommandResource(), order: CommandOrder.UPDATE_BESCHEID };

        component.ngOnInit();

        expect(component.hasBody).toBeTruthy();
      });

      it('should set false on SEND_BESCHEID', () => {
        component.command = { ...createCommandResource(), order: CommandOrder.SEND_BESCHEID };

        component.ngOnInit();

        expect(component.hasBody).toBeFalsy();
      });
    });
  });

  describe('on command with body', () => {
    beforeEach(() => {
      component.command = {
        ...createCommandResource(),
        order: CommandOrder.UPDATE_BESCHEID,
        body: createBescheid(),
      };
      component.ngOnInit();
      fixture.detectChanges();
    });

    describe('expansion panel with user', () => {
      it('should be visible', () => {
        existsAsHtmlElement(fixture, expansionPanel);
      });

      it('should be called with headline', () => {
        const expansionPanelComp: ExpansionPanelWithUserComponent =
          getMockComponent<ExpansionPanelWithUserComponent>(
            fixture,
            ExpansionPanelWithUserComponent,
          );

        expect(expansionPanelComp.headline).toBe('einen Bescheid erstellt.');
      });

      it('should be called with resource', () => {
        const expansionPanelComp: ExpansionPanelWithUserComponent =
          getMockComponent<ExpansionPanelWithUserComponent>(
            fixture,
            ExpansionPanelWithUserComponent,
          );

        expect(expansionPanelComp.resource).toBe(component.command);
      });
    });

    it('should show bewillig', () => {
      existsAsHtmlElement(fixture, status);
    });

    it('should show document', () => {
      existsAsHtmlElement(fixture, document);
    });

    describe('attachments', () => {
      describe('exists', () => {
        const attachment: string = faker.internet.url();

        beforeEach(() => {
          const bodyWithAttachments: unknown = { attachments: attachment };
          component.command = {
            ...createCommandResource(),
            order: CommandOrder.UPDATE_BESCHEID,
            body: bodyWithAttachments,
          };
          component.ngOnInit();
          fixture.detectChanges();
        });

        it('should show', () => {
          existsAsHtmlElement(fixture, attachments);
        });

        it('component should be called with attachments', () => {
          const attachmentsComp: HistorieItemAttachmentComponent =
            getMockComponent<HistorieItemAttachmentComponent>(
              fixture,
              HistorieItemAttachmentComponent,
            );

          expect(attachmentsComp.attachments).toBe(attachment);
        });
      });

      it('should hide if missing', () => {
        const bodyWithoutAttachments: unknown = { attachments: null };
        component.command = {
          ...createCommandResource(),
          order: CommandOrder.UPDATE_BESCHEID,
          body: bodyWithoutAttachments,
        };

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, attachments);
      });
    });
  });

  describe('on command without body', () => {
    const command: CommandResource = {
      ...createCommandResource(),
      order: CommandOrder.SEND_BESCHEID,
      body: null,
    };

    beforeEach(() => {
      component.command = command;
      component.ngOnInit();
      fixture.detectChanges();
    });
    it('should show header', () => {
      existsAsHtmlElement(fixture, header);
    });

    describe('item header', () => {
      it('should be called with headline', () => {
        const itemHeaderComp: HistorieItemHeaderComponent =
          getMockComponent<HistorieItemHeaderComponent>(fixture, HistorieItemHeaderComponent);

        expect(itemHeaderComp.headline).toBe('einen Bescheid per Nachricht verschickt.');
      });

      it('should be called with resource', () => {
        const itemHeaderComp: HistorieItemHeaderComponent =
          getMockComponent<HistorieItemHeaderComponent>(fixture, HistorieItemHeaderComponent);

        expect(itemHeaderComp.resource).toBe(component.command);
      });
    });
  });
});
