/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { NavigationService } from '@alfa-client/navigation-shared';
import { EMPTY_STRING, encodeUrlForEmbedding } from '@alfa-client/tech-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { VorgangHeaderLinkRel, VorgangListService, VorgangResource } from '@alfa-client/vorgang-shared';
import { fakeAsync, tick } from '@angular/core/testing';
import { FormControl, UntypedFormBuilder } from '@angular/forms';
import { Params } from '@angular/router';
import { faker } from '@faker-js/faker';
import { ResourceUri, getUrl } from '@ngxp/rest';
import { createVorgangResource } from 'libs/vorgang-shared/test/vorgang';
import { Observable, of } from 'rxjs';
import { VorgangSearchFormService } from './vorgang-search.formservice';

describe('VorgangSearchFormService', () => {
  let formService: VorgangSearchFormService;
  let navigationService: Mock<NavigationService>;
  let vorgangListService: Mock<VorgangListService>;

  const SEARCH_STRING = faker.word.noun();

  beforeEach(() => {
    vorgangListService = mock(VorgangListService);
    navigationService = mock(NavigationService);
    vorgangListService.getSearchString.mockReturnValue(of({}));

    formService = new VorgangSearchFormService(
      new UntypedFormBuilder(),
      useFromMock(navigationService),
      useFromMock(vorgangListService),
    );
  });

  it('should create', () => {
    expect(formService).toBeTruthy();
  });

  describe('setHasSearchString', () => {
    it('should set hasSearchString to true', () => {
      formService.setHasSearchString(faker.word.adverb());

      expect(formService.hasSearchString).toBeTruthy();
    });

    it('should set hasSearchString to false', () => {
      formService.setHasSearchString(EMPTY_STRING);

      expect(formService.hasSearchString).toBeFalsy();
    });
  });

  describe('subscribeToValueChanges', () => {
    it('should call setHasSearchString after debounce time', fakeAsync(() => {
      const value = faker.word.sample();
      const setHasSearchString = jest.spyOn(formService, 'setHasSearchString');

      formService.subscribeToValueChanges();
      formService.form.controls[formService.SEARCH_FIELD].patchValue(value);
      tick(300);

      expect(setHasSearchString).toHaveBeenCalledWith(value);
    }));

    it('should call handleValueChange after debounce time', fakeAsync(() => {
      const value = faker.word.sample();
      const handleValueChanges = jest.spyOn(formService, 'handleValueChanges');

      formService.subscribeToValueChanges();
      formService.form.controls[formService.SEARCH_FIELD].patchValue(value);
      tick(300);

      expect(handleValueChanges).toHaveBeenCalledWith(value);
    }));

    it('should not call handleValueChange if lastKeyPressed is Enter', fakeAsync(() => {
      const handleValueChanges = jest.spyOn(formService, 'handleValueChanges');
      formService.setLastKeyPressed('Enter');

      formService.subscribeToValueChanges();
      tick(300);

      expect(handleValueChanges).not.toHaveBeenCalled();
    }));
  });

  describe('handleValueChanges', () => {
    describe('on should search for preview', () => {
      beforeEach(() => {
        formService.shouldSearchForPreview = jest.fn().mockReturnValue(true);
      });

      it('should set searchString on at least 3 characters', () => {
        formService.handleValueChanges(SEARCH_STRING);

        expect(vorgangListService.setSearchString).toHaveBeenCalledWith(SEARCH_STRING);
      });

      it('should do search for preview list on at least 3 characters', () => {
        formService.handleValueChanges(SEARCH_STRING);

        expect(vorgangListService.searchForPreview).toHaveBeenCalledWith(SEARCH_STRING);
      });
    });

    describe('on should NOT search for preview', () => {
      beforeEach(() => {
        formService.shouldSearchForPreview = jest.fn().mockReturnValue(false);
      });

      it('should call vorgangListService.clearSearchPreviewList if search string is lesser than 3', () => {
        const searchString: string = faker.word.verb(2);

        formService.handleValueChanges(searchString);

        expect(vorgangListService.clearSearchPreviewList).toHaveBeenCalled();
      });
    });
  });

  describe('shouldSearchForPreview', () => {
    const VALID_SEARCH_STRING: string = 'minLengthSearchString';

    describe('should return false', () => {
      beforeEach(() => {
        formService.lastSearchString = 'quatsch';
      });

      it('on searchInput is pristine', () => {
        formService.isSearchInputNotPristine = jest.fn().mockReturnValue(false);

        const shouldSearch: boolean = formService.shouldSearchForPreview(VALID_SEARCH_STRING);

        expect(shouldSearch).toBeFalsy();
      });

      it('on searchString length is lower than 3', () => {
        formService.isSearchInputNotPristine = jest.fn().mockReturnValue(true);

        const shouldSearch: boolean = formService.shouldSearchForPreview('ab');

        expect(shouldSearch).toBeFalsy();
      });
    });

    describe('should return true', () => {
      beforeEach(() => {
        formService.lastSearchString = 'quatsch';
        formService.isSearchInputNotPristine = jest.fn().mockReturnValue(true);
      });

      it('on searchInput is NOT pristine and has minLength', () => {
        const shouldSearch: boolean = formService.shouldSearchForPreview(VALID_SEARCH_STRING);

        expect(shouldSearch).toBeTruthy();
      });
    });
  });

  describe('getValue', () => {
    it('should return null on empty search string', () => {
      getSearchFormControl().patchValue(EMPTY_STRING);

      const value: string = formService.getValue();

      expect(value).toBeNull();
    });

    it('should return value of search string if not nil', () => {
      getSearchFormControl().patchValue(SEARCH_STRING);

      const result: string = formService.getValue();

      expect(result).toEqual(SEARCH_STRING);
    });
  });

  describe('patchSearchString', () => {
    it('should patch search string field if its different', () => {
      const newSearchString: string = 'i search for something other...';
      getSearchFormControl().patchValue(SEARCH_STRING);

      formService.patchSearchString(newSearchString);

      expect(getSearchFormControl().value).toEqual(newSearchString);
    });

    it('should set lastSearchString', () => {
      formService.patchSearchString(SEARCH_STRING);

      expect(formService.lastSearchString).toEqual(SEARCH_STRING);
    });
  });

  describe('navigateToVorgangListOnSearch', () => {
    it('should call navigateToVorgangList when at search page', () => {
      const params: Params = { search: faker.word.sample() };

      formService.navigateToVorgangListOnSearch(params);

      expect(navigationService.navigateToVorgangList).toHaveBeenCalled();
    });

    it('should not call navigateToVorgangList when at non-search page', () => {
      const params: Params = {};

      formService.navigateToVorgangListOnSearch(params);

      expect(navigationService.navigateToVorgangList).not.toHaveBeenCalled();
    });
  });

  describe('submit', () => {
    it('should navigate to itself with search param', () => {
      formService.hasSearchString = true;
      getSearchFormControl().patchValue(SEARCH_STRING);

      formService.submit();

      expect(navigationService.search).toHaveBeenCalledWith(SEARCH_STRING);
    });
  });

  describe('clear button', () => {
    beforeEach(() => {
      navigationService.urlChanged.mockReturnValue(of({}));
    });

    it('should clear field', () => {
      getSearchFormControl().patchValue(SEARCH_STRING);

      formService.clearSearchField();

      expect(getSearchFormControl().value).toBe(EMPTY_STRING);
    });

    it('should call navigateToVorgangListOnSearch()', () => {
      formService.navigateToVorgangListOnSearch = jest.fn();

      formService.clearSearchField();

      expect(formService.navigateToVorgangListOnSearch).toHaveBeenCalled();
    });
  });

  describe('submitByPreviewList', () => {
    const vorgang: VorgangResource = createVorgangResource([VorgangHeaderLinkRel.VORGANG_WITH_EINGANG]);
    const searchString: string = faker.person.firstName();

    it('should clear searchString', () => {
      formService.submitByPreviewList(vorgang, searchString);

      expect(getSearchFormControl().value).toBe(searchString);
    });

    it('should navigate', () => {
      const uri: ResourceUri = getUrl(vorgang, VorgangHeaderLinkRel.VORGANG_WITH_EINGANG);

      formService.submitByPreviewList(vorgang, searchString);

      expect(navigationService.navigateToVorgang).toHaveBeenCalledWith(encodeUrlForEmbedding(uri));
    });
  });

  describe('isSearchInputNotPristine', () => {
    it('should return false if search INPUT is pristine', () => {
      getSearchFormControl().markAsPristine();

      expect(formService.isSearchInputNotPristine()).toBeFalsy();
    });
    it('should return true if search INPUT is not pristine', () => {
      getSearchFormControl().markAsDirty();

      expect(formService.isSearchInputNotPristine()).toBeTruthy();
    });
  });

  describe('isSameSearchString', () => {
    beforeEach(() => {
      formService.getValue = jest.fn().mockReturnValue(SEARCH_STRING);
    });

    it('should return false if search string has changed', () => {
      formService.lastSearchString = faker.word.noun();

      expect(formService.isSameSearchString()).toBeFalsy();
    });
    it('should return true if search string has NOT changed', () => {
      formService.lastSearchString = SEARCH_STRING;

      expect(formService.isSameSearchString()).toBeTruthy();
    });
  });

  describe('getValueChanges', () => {
    it.skip('should return observable for search string', () => {
      getSearchFormControl().patchValue(SEARCH_STRING);

      const result: Observable<string> = formService.getValueChanges();

      expect(result).toBe(SEARCH_STRING);
    });
  });

  function getSearchFormControl(): FormControl {
    return <FormControl>formService.form.controls[formService.SEARCH_FIELD];
  }
});
