/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa;

import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;

import org.springframework.beans.factory.ObjectFactory;
import org.springframework.beans.factory.config.Scope;
import org.springframework.core.NamedInheritableThreadLocal;

import lombok.extern.log4j.Log4j2;

@Log4j2
public class CallScope implements Scope {

	public static final String SCOPE_NAME = "call";

	private static final ThreadLocal<Map<String, Object>> scopedObjectsHolder = new NamedInheritableThreadLocal<>("Call Context");
	private static final ThreadLocal<Map<String, Runnable>> destructionCallbacksHolder = new NamedInheritableThreadLocal<>(
			"Call Context Desctruction Callbacks");

	public void startScope() {
		LOG.info("START Call-Scope");
		scopedObjectsHolder.set(new ConcurrentHashMap<>());
		destructionCallbacksHolder.set(new ConcurrentHashMap<>());
	}

	public void endScope() {
		scopedObjectsHolder.remove();
		callAllDestructionCallbacks();
		destructionCallbacksHolder.remove();
	}

	private void callAllDestructionCallbacks() {
		destructionCallbacksHolder.get().values().forEach(Runnable::run);
	}

	@Override
	public Object get(String name, ObjectFactory<?> objectFactory) {
		// This is a workaround for a problematic CallScope implementation. This should only happen during e2e tests execution.
		if (Objects.isNull(scopedObjectsHolder.get())) {
			LOG.warn("ThreadLocal value for {} not found.", name);
			return objectFactory.getObject();
		}

		if (!scopedObjectsHolder.get().containsKey(name)) {
			scopedObjectsHolder.get().put(name, objectFactory.getObject());
		}

		return scopedObjectsHolder.get().get(name);
	}

	@Override
	public Object remove(String name) {
		destructionCallbacksHolder.get().remove(name);
		return scopedObjectsHolder.get().remove(name);
	}

	@Override
	public void registerDestructionCallback(String name, Runnable callback) {
		destructionCallbacksHolder.get().put(name, callback);

	}

	@Override
	public Object resolveContextualObject(String key) {
		return scopedObjectsHolder.get().get(key);
	}

	@Override
	public String getConversationId() {
		return Thread.currentThread().getName();
	}
}
