/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.historie;

import java.util.Map;
import java.util.Optional;
import java.util.function.Predicate;

import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItem;
import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItemService;
import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandBodyMapper;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.common.command.CommandService;
import de.ozgcloud.alfa.common.command.CommandStatus;
import de.ozgcloud.alfa.common.command.LegacyOrder;
import de.ozgcloud.alfa.kommentar.Kommentar;
import de.ozgcloud.alfa.loeschanforderung.DeleteLoeschAnforderung;
import de.ozgcloud.alfa.wiedervorlage.Wiedervorlage;

/**
 * Bitte zukuenftig als Implementierung von {@link HistorieProcessor} in den
 * jeweiligen fachlichen packages umsetzen.
 *
 * TODO: Aktuellen Code auf Processoren umstellen
 */
@Component
class HistorieCommandHandler {

	static final String DIRECTION_INCOMMING = "IN";
	static final String DIRECTION_OUTGOING = "OUT";
	static final String DIRECTION = "direction";
	static final String OZGCLOUD_NACHRICHTEN_MANAGER = "OzgCloud_NachrichtenManager";
	static final String CLIENT = "client";

	private static final Predicate<Command> IS_CREATE_ATTACHED_ITEM = command -> command.getCommandOrder() == CommandOrder.CREATE_ATTACHED_ITEM;

	@Autowired
	private VorgangAttachedItemService vorgangAttachedItemService;

	@Autowired
	private CommandService commandService;

	public boolean isHistorieCommand(Command command) {
		return !isOutgoingPostfachNachrichtByOzgCloudNachrichtenManager(command)
				&& !isCreateLoeschAnforderungVorgangAttachedItem(command)
				&& !isLoeschAnforderungZuruecknehmenRevoked(command)
				&& !isDeleteVorgangAttachedItem(command);
	}

	boolean isOutgoingPostfachNachrichtByOzgCloudNachrichtenManager(Command command) {
		return command.getCommandOrder().equals(CommandOrder.CREATE_ATTACHED_ITEM) && isOzgCloudNachrichtenManager(command) && isOutgoing(command);
	}

	private boolean isOzgCloudNachrichtenManager(Command command) {
		return StringUtils.equals(MapUtils.getString(command.getBody(), CLIENT), OZGCLOUD_NACHRICHTEN_MANAGER);
	}

	private boolean isOutgoing(Command command) {
		return isDirection(command, DIRECTION_OUTGOING);
	}

	boolean isCreateLoeschAnforderungVorgangAttachedItem(Command command) {
		return Optional.of(command)
				.filter(IS_CREATE_ATTACHED_ITEM)
				.map(Command::getBody)
				.map(bodyMap -> MapUtils.getString(bodyMap, VorgangAttachedItem.FIELD_ITEM_NAME))
				.map(vorgangAttachedItemService::isLoeschAnforderung)
				.orElse(false);
	}

	boolean isLoeschAnforderungZuruecknehmenRevoked(Command command) {
		if (command.getCommandOrder() != CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN) {
			return false;
		}
		var changeStatusCommand = commandService.getById(getChangeStatusCommandId(command));
		return changeStatusCommand.getStatus() == CommandStatus.REVOKED;
	}

	private String getChangeStatusCommandId(Command command) {
		return MapUtils.getString(command.getBody(), DeleteLoeschAnforderung.CHANGE_STATUS_COMMAND_ID_FIELD);
	}

	boolean isDeleteVorgangAttachedItem(Command command) {
		return command.getCommandOrder() == CommandOrder.DELETE_ATTACHED_ITEM;
	}

	public Command translateOrder(Command command) {
		HistorieCommandHandler translator = new HistorieCommandHandler();
		return switch (command.getCommandOrder()) {
			case CREATE_ATTACHED_ITEM:
				yield translator.mapCreateOrder(command);
			case UPDATE_ATTACHED_ITEM:
				yield translator.mapUpdateOrder(command);
			case PATCH_ATTACHED_ITEM:
				yield translator.mapPatchOrder(command);
			case SEND_POSTFACH_MAIL:
				yield command.toBuilder().order(CommandOrder.SEND_POSTFACH_NACHRICHT.name()).build();
			default:
				yield command;
		};
	}

	private Command mapCreateOrder(Command command) {
		var resultBuilder = command.toBuilder();
		var itemName = getItemName(command);

		itemName.ifPresent(name -> {
			if (name.equals(Kommentar.class.getSimpleName())) {
				resultBuilder.order(LegacyOrder.CREATE_KOMMENTAR).build();
			} else if (name.equals(Wiedervorlage.class.getSimpleName())) {
				resultBuilder.order(LegacyOrder.CREATE_WIEDERVORLAGE).build();
			} else if (isOzgCloudNachrichtenManager(command) && isIncomming(command)) {
				resultBuilder.order(CommandOrder.RECEIVE_POSTFACH_NACHRICHT.name()).build();
			}
		});

		return resultBuilder.build();
	}

	private boolean isIncomming(Command command) {
		return isDirection(command, DIRECTION_INCOMMING);
	}

	private boolean isDirection(Command command, String expectedDirection) {
		return StringUtils.equals(MapUtils.getString(getItemMap(command), DIRECTION), expectedDirection);
	}

	private Command mapUpdateOrder(Command command) {
		var resultBuilder = command.toBuilder();
		var itemName = getItemName(command);

		itemName.ifPresent(name -> {
			if (name.equals(Kommentar.class.getSimpleName())) {
				resultBuilder.order(LegacyOrder.EDIT_KOMMENTAR).build();
			} else if (name.equals(Wiedervorlage.class.getSimpleName())) {
				resultBuilder.order(LegacyOrder.EDIT_WIEDERVORLAGE).build();
			}
		});

		return resultBuilder.build();
	}

	private Optional<String> getItemName(Command command) {
		return Optional.ofNullable(MapUtils.getString(command.getBody(), CommandBodyMapper.ITEM_NAME_PROPERTY));
	}

	private Command mapPatchOrder(Command command) {
		var resultBuilder = command.toBuilder();
		var isDone = getDoneValue(command);

		isDone.ifPresent(done -> {
			if (done.booleanValue()) {
				resultBuilder.order(LegacyOrder.WIEDERVORLAGE_ERLEDIGEN).build();
			} else {
				resultBuilder.order(LegacyOrder.WIEDERVORLAGE_WIEDEREROEFFNEN).build();
			}
		});

		return resultBuilder.build();
	}

	private Optional<Boolean> getDoneValue(Command command) {
		return Optional.ofNullable(MapUtils.getBoolean(getItemMap(command), Wiedervorlage.DONE_FIELD));
	}

	@SuppressWarnings("unchecked")
	private Map<String, Object> getItemMap(Command command) {
		return MapUtils.getMap(command.getBody(), CommandBodyMapper.ITEM_PROPERTY);
	}
}