/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.historie;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.util.Objects;
import java.util.Optional;
import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;
import org.springframework.hateoas.server.RepresentationModelAssembler;
import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.common.ModelBuilder;
import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.user.UserManagerUrlProvider;

@Component
class HistorieModelAssembler implements RepresentationModelAssembler<Command, EntityModel<Command>> {
	static final String SYSTEM_NOTIFICATION_MANAGER_PREFIX = "system-notification_manager";

	static final String REL_ASSIGNED_TO = "assignedTo";

	static final String ASSIGNED_TO_BODY_FIELD = "assignedTo";

	@Autowired
	private UserManagerUrlProvider userManagerUrlProvider;

	@Override
	public EntityModel<Command> toModel(Command entity) {
		var cleanCommand = unsetSystemUser(entity);
		var modelBuilder = ModelBuilder.fromEntity(cleanCommand).addLink(linkTo(HistorieController.class).slash(cleanCommand.getId()).withSelfRel());

		addAssignedTo(cleanCommand, modelBuilder);

		return modelBuilder.buildModel();
	}

	Command unsetSystemUser(Command entity) {
		if (isSystemNotificationUser(entity)) {
			return entity.toBuilder().createdBy(null).build();
		}

		return entity;
	}

	private boolean isSystemNotificationUser(Command entity) {
		return Objects.nonNull(entity.getCreatedBy()) && entity.getCreatedBy().toString().startsWith(SYSTEM_NOTIFICATION_MANAGER_PREFIX);
	}

	private void addAssignedTo(Command entity, ModelBuilder<Command> modelBuilder) {
		Optional.ofNullable(entity.getBody()).map(body -> body.get(ASSIGNED_TO_BODY_FIELD))
				.ifPresent(assignedTo -> {
					if (userManagerUrlProvider.isConfiguredForUserProfile()) {
						modelBuilder.addLink(Link.of(String.format(userManagerUrlProvider.getUserProfileTemplate(), assignedTo), REL_ASSIGNED_TO));
					}
				});
	}

	public CollectionModel<EntityModel<Command>> toCollectionModel(Stream<Command> entities) {
		return CollectionModel.of(entities.map(this::toModel).toList(), linkTo(HistorieController.class).withSelfRel());
	}
}