/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.postfach;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Stream;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.Resource;
import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.common.GermanDateTimeFormatter;
import de.ozgcloud.alfa.common.user.UserManagerUrlProvider;
import de.ozgcloud.alfa.common.user.UserProfile;
import de.ozgcloud.alfa.postfach.PostfachMail.Direction;
import de.ozgcloud.alfa.vorgang.Antragsteller;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.ozgcloud.common.errorhandling.TechnicalException;
import de.ozgcloud.common.pdf.PdfService;

@Service
class PostfachNachrichtPdfService {

	static final String PDF_TEMPLATE_PATH = "classpath:fop/postfach-nachrichten.xsl";

	static final String SYSTEM_NACHRICHT_NAME = StringUtils.EMPTY;
	static final String FALLBACK_USER_NAME = "Unbekannter Benutzer";

	static final String FALLBACK_ANTRAGSTELLER_NAME = "Antragsteller";

	@Autowired
	private PdfService pdfService;

	@Autowired
	private GermanDateTimeFormatter germanDateTimeFormatter;

	private boolean isFirstNachricht;

	@Value(PostfachNachrichtPdfService.PDF_TEMPLATE_PATH)
	private Resource pdfTemplate;

	public OutputStream getAllAsPdf(VorgangWithEingang vorgang, Stream<PostfachNachrichtPdfData> postfachNachrichten, OutputStream out) {
		return pdfService.createPdf(getTemplate(), out, buildModel(vorgang, postfachNachrichten));
	}

	InputStream getTemplate() {
		try {
			return pdfTemplate.getInputStream();
		} catch (IOException e) {
			throw new TechnicalException("Pdf Template for postfach nachrichten could not be loaded", e);
		}
	}

	PostfachNachrichtPdfModel buildModel(VorgangWithEingang vorgang, Stream<PostfachNachrichtPdfData> postfachNachrichten) {
		var pdfModelBuilder = PostfachNachrichtPdfModel.builder().vorgangNummer(vorgang.getNummer()).vorgangName(vorgang.getName());

		Optional.ofNullable(vorgang.getEingang().getAntragsteller()).ifPresent(antragsteller -> mapAntragsteller(pdfModelBuilder, antragsteller));

		mapNachrichten(pdfModelBuilder, postfachNachrichten, vorgang.getEingang().getAntragsteller());

		return pdfModelBuilder.build();
	}

	void mapAntragsteller(PostfachNachrichtPdfModel.PostfachNachrichtPdfModelBuilder modelBuilder, Antragsteller antragsteller) {
		modelBuilder.antragstellerAnrede(antragsteller.getAnrede())
				.antragstellerVorname(antragsteller.getVorname())
				.antragstellerNachname(antragsteller.getNachname())
				.antragstellerStrasse(antragsteller.getStrasse())
				.antragstellerHausnummer(antragsteller.getHausnummer())
				.antragstellerPlz(antragsteller.getPlz())
				.antragstellerOrt(antragsteller.getOrt());
	}

	private void mapNachrichten(PostfachNachrichtPdfModel.PostfachNachrichtPdfModelBuilder pdfModelBuilder,
			Stream<PostfachNachrichtPdfData> postfachNachrichten, Antragsteller antragsteller) {
		isFirstNachricht = true;

		pdfModelBuilder.nachrichten(postfachNachrichten.map(nachricht -> mapPostfachNachricht(nachricht, antragsteller)).toList());

	}

	PostfachNachrichtPdfModel.Nachricht mapPostfachNachricht(PostfachNachrichtPdfData postfachMail, Antragsteller antragsteller) {
		return PostfachNachrichtPdfModel.Nachricht.builder()
				.isFirst(isFirstNachricht())
				.subject(postfachMail.getSubject())
				.mailBody(postfachMail.getMailBody())
				.createdAt(germanDateTimeFormatter.formatZonedDateTime(postfachMail.getCreatedAt()))
				.createdBy(buildAbsenderName(postfachMail, antragsteller))
				.attachments(postfachMail.getAttachmentNames())
				.build();
	}

	private boolean isFirstNachricht() {
		if (isFirstNachricht) {
			isFirstNachricht = false;
			return true;
		}
		return false;
	}

	String buildAbsenderName(PostfachNachrichtPdfData postfachNachrichtPdfData, Antragsteller antragsteller) {
		if (postfachNachrichtPdfData.getDirection() == Direction.IN) {
			return buildAbsenderForOutgoingNachricht(antragsteller);
		}
		return buildAbsenderForIncomingNachricht(postfachNachrichtPdfData);
	}

	String buildAbsenderForOutgoingNachricht(Antragsteller antragsteller) {
		return Optional.ofNullable(antragsteller)
				.map(this::formatAntragstellerName)
				.orElseGet(() -> FALLBACK_ANTRAGSTELLER_NAME);
	}

	String buildAbsenderForIncomingNachricht(PostfachNachrichtPdfData postfachNachrichtPdfData) {
		return Optional.ofNullable(postfachNachrichtPdfData.getUser()).map(user -> {
			if (Objects.nonNull(user.getId()) && user.toString().startsWith(UserManagerUrlProvider.SYSTEM_USER_IDENTIFIER)) {
				return SYSTEM_NACHRICHT_NAME;
			}
			return formatUserName(postfachNachrichtPdfData.getUser());
		}).orElseGet(() -> FALLBACK_USER_NAME);
	}

	private String formatUserName(UserProfile user) {
		return formatForPdf(user.getFirstName(), user.getLastName());
	}

	private String formatAntragstellerName(Antragsteller antragsteller) {
		return formatForPdf(antragsteller.getVorname(), antragsteller.getNachname());
	}

	private String formatForPdf(String firstName, String lastName) {
		return String.format("%s %s", getPdfStringValue(firstName), getPdfStringValue(lastName)).trim();
	}

	private String getPdfStringValue(String value) {
		return Optional.ofNullable(StringUtils.trimToNull(value)).orElseGet(() -> StringUtils.EMPTY);
	}
}