package de.ozgcloud.alfa.statistic;

import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.AlfaProperties;
import de.ozgcloud.alfa.common.GrpcUtil;
import de.ozgcloud.alfa.vorgang.Vorgang.VorgangStatus;
import de.ozgcloud.vorgang.common.GrpcQueryOperator;
import de.ozgcloud.vorgang.statistic.GrpcVorgangStatisticQuery;
import de.ozgcloud.vorgang.statistic.GrpcVorgangStatisticQuery.GroupMethod;
import de.ozgcloud.vorgang.statistic.GrpcVorgangStatisticRequest;
import de.ozgcloud.vorgang.statistic.GrpcVorgangStatisticResponse;
import de.ozgcloud.vorgang.statistic.StatisticServiceGrpc.StatisticServiceBlockingStub;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service
class StatisticRemoteService {

	static final String COUNT_VORGANG_STATUS_RESULT_NAME_FORMAT = "countVorgangStatus_%s";
	static final String COUNT_WIEDERVORLAGE_NEXT_FRIST_RESULT_NAME = "countWiedervorlage";
	static final String EXISTS_WIEDERVORLAGE_OVERDUE_RESULT_NAME = "existsWiedervorlageOverdue";
	static final String COUNT_VORGAENGE_WITH_UNGELESENE_NACHRICHTEN_RESULT_NAME = "countVorgaengeWithUngeleseneNachrichten";

	static final String STATUS_PATH = "Vorgang.status";
	static final String WIEDERVORLAGE_NEXT_FRIST_PATH_TEMPLATE = "ClientAttribute.%s.nextWiedervorlageFrist";
	static final String VORGAENGE_WITH_UNGELESENE_NACHRICHTEN_PATH = "ClientAttribute.OzgCloud_NachrichtenManager.hasNewPostfachNachricht";

	static final String OPERAND_TODAY_DATE = "today()";

	@GrpcClient(GrpcUtil.VORGANG_MANAGER_GRPC_CLIENT)
	private StatisticServiceBlockingStub statisticServiceStub;
	@Autowired
	private StatisticMapper mapper;
	@Autowired
	private AlfaProperties alfaProperties;

	public Map<String, StatisticResult> getVorgaengeStatistics(List<VorgangStatus> countByVorgangStatus) {
		var grpcRequest = GrpcVorgangStatisticRequest.newBuilder()
				.addAllQuery(buildCountByStatusQueries(countByVorgangStatus))
				.addQuery(buildCountWiedervorlageNextFristQuery())
				.addQuery(buildExistsWiedervorlageOverdueQuery())
				.addQuery(buildCountVorgaengeWithUngeleseneNachrichtenQuery())
				.build();

		var grpcResponse = statisticServiceStub.getVorgangStatistic(grpcRequest);

		return buildStatisticResult(grpcResponse);
	}

	List<GrpcVorgangStatisticQuery> buildCountByStatusQueries(List<VorgangStatus> countByVorgangStatus) {
		return countByVorgangStatus.stream().map(this::buildCountByVorgangStatusQuery).toList();
	}

	GrpcVorgangStatisticQuery buildCountByVorgangStatusQuery(VorgangStatus vorgangStatus) {
		return GrpcVorgangStatisticQuery.newBuilder()
				.setResultName(buildCountByStatusResultName(vorgangStatus))
				.setPath(STATUS_PATH)
				.setGroupMethod(GroupMethod.COUNT)
				.setOperator(GrpcQueryOperator.EQUAL)
				.setOperandStringValue(vorgangStatus.name())
				.build();
	}

	GrpcVorgangStatisticQuery buildCountWiedervorlageNextFristQuery() {
		return GrpcVorgangStatisticQuery.newBuilder()
				.setResultName(COUNT_WIEDERVORLAGE_NEXT_FRIST_RESULT_NAME)
				.setPath(String.format(WIEDERVORLAGE_NEXT_FRIST_PATH_TEMPLATE, alfaProperties.getApplicationName()))
				.setGroupMethod(GroupMethod.COUNT)
				.setOperator(GrpcQueryOperator.UNEQUAL)
				.build();
	}

	GrpcVorgangStatisticQuery buildExistsWiedervorlageOverdueQuery() {
		return GrpcVorgangStatisticQuery.newBuilder()
				.setResultName(EXISTS_WIEDERVORLAGE_OVERDUE_RESULT_NAME)
				.setPath(String.format(WIEDERVORLAGE_NEXT_FRIST_PATH_TEMPLATE, alfaProperties.getApplicationName()))
				.setGroupMethod(GroupMethod.EXISTS)
				.setOperator(GrpcQueryOperator.LESS_THEN_OR_EQUAL_TO)
				.setOperandStringValue(OPERAND_TODAY_DATE)
				.build();
	}

	public String buildCountByStatusResultName(VorgangStatus vorgangStatus) {
		return String.format(COUNT_VORGANG_STATUS_RESULT_NAME_FORMAT, vorgangStatus.name());
	}

	Map<String, StatisticResult> buildStatisticResult(GrpcVorgangStatisticResponse grpcVorgangStatisticResponse) {
		return grpcVorgangStatisticResponse.getResultList()
				.stream()
				.map(mapper::toResult)
				.collect(Collectors.toMap(StatisticResult::getName, Function.identity()));
	}

	public GrpcVorgangStatisticQuery buildCountVorgaengeWithUngeleseneNachrichtenQuery() {
		return GrpcVorgangStatisticQuery.newBuilder()
				.setResultName(COUNT_VORGAENGE_WITH_UNGELESENE_NACHRICHTEN_RESULT_NAME)
				.setPath(VORGAENGE_WITH_UNGELESENE_NACHRICHTEN_PATH)
				.setGroupMethod(GroupMethod.COUNT)
				.setOperator(GrpcQueryOperator.EQUAL)
				.setOperandBoolValue(true)
				.build();
	}
}