/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.callcontext;

import static de.ozgcloud.alfa.common.callcontext.ContextService.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Collection;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.context.ApplicationContext;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.alfa.RequestAttributes;
import de.ozgcloud.alfa.RequestAttributesTestFactory;
import de.ozgcloud.alfa.common.GrpcUtil;
import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserProfileTestFactory;
import de.ozgcloud.alfa.common.user.UserRole;
import de.ozgcloud.alfa.vorgang.ZustaendigeStelleTestFactory;
import de.ozgcloud.vorgang.grpc.command.GrpcUser;

class ContextServiceTest {

	static final String APPLICATION_ID = LoremIpsum.getInstance().getName();

	@InjectMocks
	private ContextService service;

	@Mock
	private CurrentUserService userService;
	@Mock
	private ApplicationContext context;
	@Spy
	private RequestAttributes reqAttributes = RequestAttributesTestFactory.create();

	@BeforeEach
	void initMocks() {
		when(context.getId()).thenReturn(APPLICATION_ID);
		when(userService.getUser()).thenReturn(UserProfileTestFactory.create());
	}

	@DisplayName("Get context metas")
	@Nested
	class TestGetContextMetas {

		@BeforeEach
		void mockUserService() {
			when(userService.hasRole(any())).thenReturn(Boolean.TRUE);
		}

		@Test
		void shouldHaveUserId() {
			var metadata = service.buildCallContextMetadata();

			assertThat(GrpcUtil.getFromHeaders(KEY_USER_ID, metadata)).isEqualTo(UserProfileTestFactory.ID.toString());
		}

		@Test
		void shouldHaveUserName() {
			var metadata = service.buildCallContextMetadata();

			assertThat(GrpcUtil.getFromHeaders(KEY_USER_NAME, metadata)).isEqualTo(UserProfileTestFactory.FULLNAME);
		}

		@Test
		void shouldHaveClientName() {
			var metadata = service.buildCallContextMetadata();

			assertThat(GrpcUtil.getFromHeaders(KEY_CLIENT_NAME, metadata)).isEqualTo(APPLICATION_ID);
		}

		@Test
		void shouldLimitAccessToOrgaId() {
			var metadata = service.buildCallContextMetadata();
			metadata.put(GrpcUtil.createKeyOf(KEY_ACCESS_LIMITED_ORGAID), "orgaid_2".getBytes());

			assertThat(GrpcUtil.getCollection(KEY_ACCESS_LIMITED_ORGAID, metadata)).isInstanceOf(Collection.class).hasSize(2)
					.contains(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEITEN_ID, "orgaid_2");
		}

		@DisplayName("access limited")
		@Nested
		class TestAccessLimitedByUserRole {

			@Test
			void shouldHaveTrueOnRoleVerwaltungUser() {
				when(userService.hasRole(anyString())).thenReturn(Boolean.FALSE);

				var metadata = service.buildCallContextMetadata();

				assertThat(GrpcUtil.getFromHeaders(KEY_ACCESS_LIMITED, metadata)).isEqualTo(Boolean.TRUE.toString());
			}

			@Test
			void shouldHaveFalseOnRoleVerwaltungPoststelle() {
				when(userService.hasRole(UserRole.VERWALTUNG_POSTSTELLE)).thenReturn(Boolean.TRUE);

				var metadata = service.buildCallContextMetadata();

				assertThat(GrpcUtil.getFromHeaders(KEY_ACCESS_LIMITED, metadata)).isEqualTo(Boolean.FALSE.toString());
			}

			@Test
			void shouldHaveFalseOnRoleEinheitlicherAnsprechpartner() {
				when(userService.hasRole(UserRole.VERWALTUNG_POSTSTELLE)).thenReturn(Boolean.FALSE);
				when(userService.hasRole(UserRole.EINHEITLICHER_ANSPRECHPARTNER)).thenReturn(Boolean.TRUE);

				var metadata = service.buildCallContextMetadata();

				assertThat(GrpcUtil.getFromHeaders(KEY_ACCESS_LIMITED, metadata)).isEqualTo(Boolean.FALSE.toString());
			}
		}
	}

	@Nested
	class TestCreateCallContest {

		@Test
		void shouldCallContextGetApplicationName() {
			service.createCallContext();

			verify(context).getId();
		}

		@Test
		void shouldContainClient() {
			var callContext = service.createCallContext();

			assertThat(callContext.getClient()).isEqualTo(APPLICATION_ID);
		}

		@Test
		void shouldCallUserService() {
			service.createCallContext();

			verify(userService).getUser();
		}

		@Test
		void shoultHaveUserId() {
			var context = service.createCallContext();

			assertThat(context.getUser()).extracting(GrpcUser::getId).isEqualTo(UserProfileTestFactory.ID.toString());
		}

		@Test
		void shouldHaveFullName() {
			var context = service.createCallContext();

			assertThat(context.getUser()).extracting(GrpcUser::getName).isEqualTo(UserProfileTestFactory.FULLNAME);
		}

		@Test
		void shouldHaveRequestId() {
			var context = service.createCallContext();

			assertThat(context.getRequestId()).isEqualTo(RequestAttributesTestFactory.REQUEST_ID);
		}
	}
}