package de.ozgcloud.alfa.export;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Iterator;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.MockedStatic;

import com.google.protobuf.ByteString;
import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.archive.grpc.export.GrpcExportVorgangResponse;
import de.ozgcloud.common.errorhandling.TechnicalException;

class StreamedExportedVorgangFileTest {

	@Mock
	private Iterator<GrpcExportVorgangResponse> responseIterator;

	@Nested
	class TestContructor {

		private MockedStatic<StreamedExportedVorgangFile> mockedStatic;

		@BeforeEach
		void init() {
			mockedStatic = mockStatic(StreamedExportedVorgangFile.class);
			mockedStatic.when(() -> StreamedExportedVorgangFile.getFileNameFrom(responseIterator)).thenReturn(GrpcFileTestFactory.FILE_NAME);
		}

		@AfterEach
		void cleanup() {
			mockedStatic.close();
		}

		@Test
		void shouldGetFileNameFromResponseIterator() {
			new StreamedExportedVorgangFile(responseIterator);

			mockedStatic.verify(() -> StreamedExportedVorgangFile.getFileNameFrom(responseIterator));
		}

		@Test
		void shouldSetFileName() {
			var exportedVorgangFile = new StreamedExportedVorgangFile(responseIterator);

			assertThat(exportedVorgangFile.getFileName()).isEqualTo(GrpcFileTestFactory.FILE_NAME);
		}
	}

	@Nested
	class TestGetFileNameFrom {

		@Nested
		class OnEmptyResponse {

			@BeforeEach
			void init() {
				when(responseIterator.hasNext()).thenReturn(false);
			}

			@Test
			void shouldThrowException() {
				assertThatExceptionOfType(TechnicalException.class).isThrownBy(TestGetFileNameFrom.this::callTestedMethod);
			}
		}

		@Nested
		class OnNotEmptyResponse {

			@BeforeEach
			void init() {
				when(responseIterator.hasNext()).thenReturn(true);
				when(responseIterator.next()).thenReturn(GrpcExportVorgangResponseTestFactory.createWithName());
			}

			@Test
			void shouldCallHasNext() {
				callTestedMethod();

				verify(responseIterator).hasNext();
			}

			@Test
			void shouldCallNextOnce() {
				callTestedMethod();

				verify(responseIterator, times(1)).next();
			}

			@Test
			void shouldReturnFileName() {
				var fileName = callTestedMethod();

				assertThat(fileName).isEqualTo(GrpcFileTestFactory.FILE_NAME);
			}
		}

		private String callTestedMethod() {
			return StreamedExportedVorgangFile.getFileNameFrom(responseIterator);
		}
	}

	@Nested
	class TestWriteToOutputStream {

		public static final ByteString FILE_CONTENT_1 = GrpcFileTestFactory.FILE_CONTENT;
		public static final ByteString FILE_CONTENT_2 = ByteString.copyFromUtf8(LoremIpsum.getInstance().getWords(8));

		@Mock
		private OutputStream outputStream;

		private MockedStatic<StreamedExportedVorgangFile> mockedStatic;

		@BeforeEach
		void init() {
			mockedStatic = mockStatic(StreamedExportedVorgangFile.class);
			mockedStatic.when(() -> StreamedExportedVorgangFile.getFileNameFrom(responseIterator)).thenReturn(GrpcFileTestFactory.FILE_NAME);
			when(responseIterator.hasNext())
					.thenReturn(true)
					.thenReturn(true)
					.thenReturn(false);
			when(responseIterator.next())
					.thenReturn(GrpcExportVorgangResponseTestFactory.createWithContent())
					.thenReturn(GrpcExportVorgangResponseTestFactory.createBuilder().setVorgangFile(
							GrpcFileTestFactory.createBuilder().setFileContent(FILE_CONTENT_2).build()
					).build());
		}

		@AfterEach
		void cleanup() {
			mockedStatic.close();
		}

		@Test
		void shouldWriteFileContentInOrder() throws IOException {
			var orderVerifier = inOrder(outputStream);

			new StreamedExportedVorgangFile(responseIterator).writeToOutputStream(outputStream);

			orderVerifier.verify(outputStream).write(FILE_CONTENT_1.toByteArray());
			orderVerifier.verify(outputStream).write(FILE_CONTENT_2.toByteArray());
		}
	}
}
