package de.ozgcloud.alfa.resource;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentMatcher;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.RepresentationModel;

import de.ozgcloud.alfa.common.errorhandling.ResourceNotFoundException;
import de.ozgcloud.common.errorhandling.TechnicalException;
import lombok.SneakyThrows;

class OzgcloudResourceControllerTest {

	@Mock
	private OzgcloudResourceURIResolver resourceAResolver;
	@Mock
	private OzgcloudResourceURIResolver resourceBResolver;
	@Spy
	private Collection<OzgcloudResourceURIResolver> resourceResolvers = new ArrayList<>();
	@Mock
	private OzgcloudResourceModelAssembler assembler;
	@InjectMocks
	private OzgcloudResourceController controller;

	@BeforeEach
	void init() {
		resourceResolvers.add(resourceAResolver);
		resourceResolvers.add(resourceBResolver);
	}

	@Nested
	class TestGetOzgcloudResource {

		@Nested
		class OnUriCouldBeResolved {

			private final ArgumentMatcher<OzgcloudResourceURIResolveResult> HAS_MATCHING_RESOURCE_URI = mapping -> mapping.getResourceURI().equals(
					OzgcloudResourceURIResolveResultTestFactory.RESOURCE_URI);
			private final ArgumentMatcher<OzgcloudResourceURIResolveResult> HAS_MATCHING_RESOURCE_LINK = mapping -> mapping.getResourceLink().equals(
					OzgcloudResourceURIResolveResultTestFactory.RESOURCE_LINK);

			@BeforeEach
			void init() {
				when(resourceBResolver.resolve(OzgcloudResourceURIResolveResultTestFactory.RESOURCE_URI)).thenReturn(Optional.of(
						OzgcloudResourceURIResolveResultTestFactory.RESOURCE_LINK));
			}

			@Test
			void shouldCallResolver() {
				callController();

				verify(resourceBResolver).resolve(OzgcloudResourceURIResolveResultTestFactory.RESOURCE_URI);
			}

			@Test
			void shouldCallAssemblerWithUri() {
				callController();

				verify(assembler).toModel(argThat(HAS_MATCHING_RESOURCE_URI));
			}

			@Test
			void shouldCallAssemblerWithResourceLink() {
				callController();

				verify(assembler).toModel(argThat(HAS_MATCHING_RESOURCE_LINK));
			}

			@Test
			void shouldReturnModelFromAssembler() {
				EntityModel<OzgcloudResource> modelFromAssembler = EntityModel.of(new OzgcloudResource());
				when(assembler.toModel(argThat(mapping -> HAS_MATCHING_RESOURCE_URI.matches(mapping) && HAS_MATCHING_RESOURCE_LINK.matches(mapping))))
						.thenReturn(modelFromAssembler);

				var model = callController();

				assertThat(model).isEqualTo(modelFromAssembler);
			}
		}

		@Nested
		class OnUriCouldNotBeResolved {

			@BeforeEach
			void init() {
				when(resourceAResolver.resolve(OzgcloudResourceURIResolveResultTestFactory.RESOURCE_URI)).thenReturn(Optional.empty());
				when(resourceBResolver.resolve(OzgcloudResourceURIResolveResultTestFactory.RESOURCE_URI)).thenReturn(Optional.empty());
			}

			@Test
			void shouldThrowResourceNotFoundException() {
				assertThatThrownBy(TestGetOzgcloudResource.this::callController).isInstanceOf(ResourceNotFoundException.class);
			}
		}

		@Nested
		class OnMultipleResolveResults {

			@BeforeEach
			void init() {
				when(resourceAResolver.resolve(OzgcloudResourceURIResolveResultTestFactory.RESOURCE_URI)).thenReturn(Optional.of(
						OzgcloudResourceURIResolveResultTestFactory.RESOURCE_LINK));
				when(resourceBResolver.resolve(OzgcloudResourceURIResolveResultTestFactory.RESOURCE_URI)).thenReturn(Optional.of(
						OzgcloudResourceURIResolveResultTestFactory.RESOURCE_LINK));
			}

			@Test
			void shouldThrowTechnicalException() {
				assertThatThrownBy(TestGetOzgcloudResource.this::callController).isInstanceOf(TechnicalException.class);
			}
		}

		@SneakyThrows
		private RepresentationModel<EntityModel<OzgcloudResource>> callController() {
			return controller.getOzgcloudResource(OzgcloudResourceURIResolveResultTestFactory.RESOURCE_URI);
		}
	}
}
